import get from 'lodash/get';
import React from 'react';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var selectValues = function (selection, data, props) {
    if (selection === void 0) { selection = []; }
    if (data === void 0) { data = {}; }
    if (props === void 0) { props = {}; }
    if (selection instanceof Array) {
        return selectFromArray(selection, data);
    }
    if (typeof selection === 'object') {
        return selectFromObject(selection, data, props);
    }
    if (typeof selection === 'function') {
        return selection(data, props);
    }
    throw 'Mapper should be plain object, array or function';
};
function selectFromArray(selection, data) {
    if (data === void 0) { data = {}; }
    var selectionMap = {};
    selection.forEach(function (propName) {
        var _a = propName.split(':'), key = _a[0], value = _a[1];
        if (value) {
            selectionMap[key] = get(data, value);
        }
        else {
            selectionMap[key] = data[key];
        }
    });
    return selectionMap;
}
function selectFromObject(selection, data, props) {
    var selectionMap = {};
    Object.entries(selection).forEach(function (_a) {
        var key = _a[0], value = _a[1];
        if (typeof value === 'string') {
            selectionMap[key] = get(data, value);
        }
        else {
            selectionMap[key] = value(data, props);
        }
    });
    return selectionMap;
}

function connectContext(Context, Component, options) {
    if (options === void 0) { options = {}; }
    return React.memo(function (props) {
        var selection = getContextSelection(Context, options, props);
        return React.createElement(Component, __assign({}, selection, props));
    });
}
function connectContextFactory(Context) {
    return function (Component, options) {
        if (options === void 0) { options = {}; }
        return (connectContext(Context, Component, options));
    };
}
function useConnectedContextFactory(Context) {
    return function (options) {
        if (options === void 0) { options = {}; }
        var context = React.useContext(Context);
        return React.useMemo(function () {
            return getContextSelection(Context, options);
            // @ts-ignore
        }, [context.state]);
    };
}
function getContextSelection(Context, _a, props) {
    var _b = _a === void 0 ? {} : _a, stateMappers = _b.stateMappers, actionMappers = _b.actionMappers;
    if (props === void 0) { props = {}; }
    var context = React.useContext(Context);
    var state = selectValues(stateMappers, context.state, props);
    var actions = selectValues(actionMappers, context.actions, props);
    return __assign({}, state, actions);
}

function createContextProvider(reducer, initialState, actionCreators) {
    var Context = React.createContext({});
    return [
        function (props) {
            var _a = React.useReducer(reducer, initialState), state = _a[0], dispatch = _a[1];
            var actions = getBindedActions(actionCreators, dispatch);
            var value = {
                state: state,
                actions: actions,
            };
            return (React.createElement(Context.Provider, __assign({ value: value }, props)));
        },
        Context,
    ];
}
function getBindedActions(actions, dispatch) {
    var bindedActions = {};
    Object.entries(actions).forEach(function (_a) {
        var key = _a[0], value = _a[1];
        bindedActions[key] = value(dispatch);
    });
    return bindedActions;
}

export default createContextProvider;
export { connectContext, connectContextFactory, useConnectedContextFactory };
//# sourceMappingURL=index.es.js.map
